# Todo List

Es soll eine Todo-Liste erstellt werden, um Aufgaben zu verwalten.

### Motivation
Standardaufgabe bei jedem JS-Framework (wie Hallo Welt).

---
Die Aufgabe besteht aus 2 Teilaufgaben

* Erstellen eines Frontends
* Erstellen eines Backends

Es ist auch möglich, dass nur eine Teilaufgabe bearbeitet wird!

**Jegliche Kommunikation zwischen Frontend und Backend soll über JSON erfolgen, falls nötig.**

## Frontend

**Das Frontend soll mittels dem JS-Framework Vue.js erstellt werden (Version 1.x).  
Für das Design soll das Framework Semantic UI verwendet werden (css reicht).  

Grundsätzlich soll JavasScript (oder TypeScript) verwendet werden. Bei TypeScript mit extenernen Bibliotheken (z.b. jQuery) kann es mit den Typen zu Problemen kommen, da die meisten Typ-Dateien im Modul-Format sind. Man kann TypeScript auch nur für eigene Typen verwenden, falls man aber eine Variable benutzt, die keinen Typ hat (z.b. $, da es ohne Typ-Datei unbekannt ist), wird das rot unterstrichen. Umgehen kann man das, indem man mittels `declare` die Variable TypeScript bekannt macht, allerdings bekommt man dann keine autovervollständigung (sofern man bei declare den typ any verwendet).

* Tipp: Bei manchen Editoren/Ides muss node.js zusätzlich installier werden, um TypeScript zu JavaScript zu übersetzen.*

*Tipp zu TypeScript und Vue.js: Das Gerüst der Komponenten zunächst in JS schreiben, dann die einzelnen Funktionen mit TypeScript "Typen" anreichern.*

Im Frontend soll eine Todo-Liste dargestellt werden.
Die Todo-Liste besteht aus mehreren Todo-Items.
Jedes Todo-Item hat dabei eine Id, einen Namen, eine Priorität, einen Status (erledigt/offen) und einen Text.

Die `Id` ist dabei eine Zahl und einzigartig über alle Items.  
Der `Name` stellt den Namen des Items dar (string).  
Die `Priorität` liegt im Zahlenraum zwischen 0 und 10 (je höher die Zahl, desto wichtiger ist das Item) (int).  
Der `Status` kann entweder offen (nicht erledigt) oder erledigt sein (bool). Ist ein Item erledigt, sollte das visuell ersichtlich sein.  
der `Text` des Items gibt eine genauere Beschreibung des Items an und ist optional. 


Funktionen sind:

* Darstellen der Todo-Items
* Filtern der Liste nach
  * Status (Alle, nur offene, nur erledigte)
  * Name (aufsteigend und absteigend)
* Hinzufügen eines Items
* Löschen eines items
* Ändern der Reihenfolge der Items
* Ändern des Namens eines Items
* Ändern des Status eines Items
* Ändern des Textes eines Items
* Exportieren/Importieren der Items ins JSON-Format (das Ergebnis kann in einer Textbox angezeigt werden, kein Download/Datei)

Falls kein Backend erstellt wird, müssen die Items nur temporär für die aktuelle Sitzung gespeichert werden (z.B. in einer Variable).

Als Editor empfiehlt sich hier WebStorm/IntelliJ Ultimate (diese haben einen TypeScript Kompiler dabei).

## Backend

Das backend soll mittels Asp.net Core erstellt werden. Da es bei Asp.net Core gerade umstrukturierungen gibt, soll das vorhandene Projekt als Vorgabe benutzt werden.

Das Backend soll zudem Daten ausschließlich in JSON zurückgeben. D.h. die MVC Fähigkeiten von Asp.net Core sollen nicht benutzt werden.

Funktionen sind:

* Alle Items zurückgeben
* Item erstellen
* Item ändern (Name, Status, ... alle Eigenschaften von Item, außer Id)
* Item löschen
* Alle Items gefiltert nach Status (Alle, nur offene, nur erledigte) und Name (aufsteigend und absteigend) zurückgeben
* Ändern der Reihenfolge der Items (kan auch irgendwie über das Frontend erledigt werden)

Das Speichern der Daten kann mittels eine Klasse/Variable erfolgen, d.h. es muss keine Persistente Speicherung erfolgen.

Es ist natürlich auch möglich, dass man das EntityFramework Core benutzt und den InMemory-Provider. Eine Anbindung an eine echte Datenbank ist natürlich auch möglich.
Hierzu sei gesagt, dass beim EntityFramework die Konfiguration relativ schwierig werden kann...

Als Editor ist hier für Windows Visual Studio (z.B. Community-Edition ab 2015) geeignet. Für andere Betriebssysteme ist Visual Studio Code zu empfehlen (gibt es natürlich auch für Windows).

Falls die Konfiguration mit Visual Studio Code nich automatisch eingerichtet wird (links der Käfer und auf den grünen Pfeil), dann sollte man das Projekt nicht an der Wurzel öffnen **AspTest/** sondern den tieferen Ordner **AspTest/src/AspTest** und erneut Konfigurieren (links der Käfer und auf den grünen Pfeil).

Für die .net Core Version (welches installiert werden muss) sollte 1.x verwendet werden (bzw. 1.0.0-preview2-003156 funktioniert sicher).